@@ -1,8 +1,9 @@
 use crate::lang::SgLang;
-use ast_grep_config::{RuleConfig, Severity};
+use ast_grep_config::{LabelStyle, Metadata, RuleConfig, Severity};
+use ast_grep_core::Doc;
 use ast_grep_core::{meta_var::MetaVariable, tree_sitter::StrDoc, Node as SgNode};
 
-type Node<'a, L> = SgNode<'a, StrDoc<L>>;
+type Node<'t, L> = SgNode<'t, StrDoc<L>>;
 
 use std::collections::HashMap;
 
@@ -37,16 +38,18 @@ struct Range {
 
 #[derive(Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-struct LabelJSON<'a> {
-  text: &'a str,
+struct MatchNode<'t> {
+  text: Cow<'t, str>,
   range: Range,
 }
-
 #[derive(Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-struct MatchNode<'a> {
-  text: Cow<'a, str>,
+struct MatchLabel<'t, 'r> {
+  text: Cow<'t, str>,
   range: Range,
+  #[serde(skip_serializing_if = "Option::is_none")]
+  message: Option<&'r str>,
+  style: LabelStyle,
 }
 
 /// a sub field of leading and trailing text count around match.
@@ -61,8 +64,8 @@ struct CharCount {
 
 #[derive(Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-struct MatchJSON<'a, 'b> {
-  text: Cow<'a, str>,
+struct MatchJSON<'t, 'b> {
+  text: Cow<'t, str>,
   range: Range,
   file: Cow<'b, str>,
   lines: String,
@@ -73,17 +76,17 @@ struct MatchJSON<'a, 'b> {
   replacement_offsets: Option<std::ops::Range<usize>>,
   language: SgLang,
   #[serde(skip_serializing_if = "Option::is_none")]
-  meta_variables: Option<MetaVariables<'a>>,
+  meta_variables: Option<MetaVariables<'t>>,
 }
 
 #[derive(Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-struct MetaVariables<'a> {
-  single: HashMap<String, MatchNode<'a>>,
-  multi: HashMap<String, Vec<MatchNode<'a>>>,
+struct MetaVariables<'t> {
+  single: HashMap<String, MatchNode<'t>>,
+  multi: HashMap<String, Vec<MatchNode<'t>>>,
   transformed: HashMap<String, String>,
 }
-fn from_env<'a>(nm: &NodeMatch<'a>) -> Option<MetaVariables<'a>> {
+fn from_env<'t>(nm: &NodeMatch<'t>) -> Option<MetaVariables<'t>> {
   let env = nm.get_env();
   let mut vars = env.get_matched_variables().peekable();
   vars.peek()?;
@@ -145,8 +148,8 @@ fn get_range(n: &Node<'_, SgLang>) -> Range {
   }
 }
 
-impl<'a, 'b> MatchJSON<'a, 'b> {
-  fn new(nm: NodeMatch<'a>, path: &'b str, context: (u16, u16)) -> Self {
+impl<'t, 'b> MatchJSON<'t, 'b> {
+  fn new(nm: NodeMatch<'t>, path: &'b str, context: (u16, u16)) -> Self {
     let display = nm.display_context(context.0 as usize, context.1 as usize);
     let lines = format!("{}{}{}", display.leading, display.matched, display.trailing);
     MatchJSON {
@@ -165,44 +168,69 @@ impl<'a, 'b> MatchJSON<'a, 'b> {
     }
   }
 
-  fn diff(diff: Diff<'a>, path: &'b str, context: (u16, u16)) -> Self {
+  fn diff(diff: Diff<'t>, path: &'b str, context: (u16, u16)) -> Self {
     let mut ret = Self::new(diff.node_match, path, context);
     ret.replacement = Some(diff.replacement);
     ret.replacement_offsets = Some(diff.range);
     ret
   }
 }
-fn get_labels<'a>(nm: &NodeMatch<'a>) -> Option<Vec<MatchNode<'a>>> {
-  let env = nm.get_env();
-  let labels = env.get_labels("secondary")?;
-  Some(
-    labels
-      .iter()
-      .map(|l| MatchNode {
-        text: l.text(),
-        range: get_range(l),
-      })
-      .collect(),
-  )
+fn get_labels<'b, 't>(rule: &'b RuleConfig<SgLang>, nm: &NodeMatch<'t>) -> Vec<MatchLabel<'t, 'b>> {
+  rule
+    .get_labels(nm)
+    .into_iter()
+    .map(|label| {
+      let start_pos = label.start_node.start_pos();
+      let end_pos = label.end_node.end_pos();
+      let byte_offset = label.range();
+      let range = Range {
+        byte_offset: byte_offset.clone(),
+        start: Position {
+          line: start_pos.line(),
+          // TODO:using pos.column with non-matching node is Okay now
+          // because it only uses the source of the root doc
+          column: start_pos.column(nm),
+        },
+        end: Position {
+          line: end_pos.line(),
+          column: end_pos.column(nm),
+        },
+      };
+      let source = nm.get_doc().get_source();
+      MatchLabel {
+        text: Cow::Borrowed(&source[byte_offset]),
+        range,
+        message: label.message,
+        style: label.style,
+      }
+    })
+    .collect()
 }
 
 #[derive(Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-struct RuleMatchJSON<'a, 'b> {
+struct RuleMatchJSON<'t, 'b> {
   #[serde(flatten)]
-  matched: MatchJSON<'a, 'b>,
+  matched: MatchJSON<'t, 'b>,
   rule_id: &'b str,
   severity: Severity,
   note: Option<String>,
   message: String,
+  #[serde(skip_serializing_if = "Vec::is_empty")]
+  labels: Vec<MatchLabel<'t, 'b>>,
   #[serde(skip_serializing_if = "Option::is_none")]
-  labels: Option<Vec<MatchNode<'a>>>,
+  metadata: Option<Cow<'b, Metadata>>,
 }
-impl<'a, 'b> RuleMatchJSON<'a, 'b> {
-  fn new(nm: NodeMatch<'a>, path: &'b str, rule: &'b RuleConfig<SgLang>) -> Self {
+impl<'t, 'b> RuleMatchJSON<'t, 'b> {
+  fn new(nm: NodeMatch<'t>, path: &'b str, rule: &'b RuleConfig<SgLang>, metadata: bool) -> Self {
     let message = rule.get_message(&nm);
-    let labels = get_labels(&nm);
+    let labels = get_labels(rule, &nm);
     let matched = MatchJSON::new(nm, path, (0, 0));
+    let metadata = if metadata {
+      rule.metadata.as_ref().map(Cow::Borrowed)
+    } else {
+      None
+    };
     Self {
       matched,
       rule_id: &rule.id,
@@ -210,13 +238,19 @@ impl<'a, 'b> RuleMatchJSON<'a, 'b> {
       note: rule.note.clone(),
       message,
       labels,
+      metadata,
     }
   }
-  fn diff(diff: Diff<'a>, path: &'b str, rule: &'b RuleConfig<SgLang>) -> Self {
+  fn diff(diff: Diff<'t>, path: &'b str, rule: &'b RuleConfig<SgLang>, metadata: bool) -> Self {
     let nm = &diff.node_match;
     let message = rule.get_message(nm);
-    let labels = get_labels(nm);
+    let labels = get_labels(rule, nm);
     let matched = MatchJSON::diff(diff, path, (0, 0));
+    let metadata = if metadata {
+      rule.metadata.as_ref().map(Cow::Borrowed)
+    } else {
+      None
+    };
     Self {
       matched,
       rule_id: &rule.id,
@@ -224,6 +258,7 @@ impl<'a, 'b> RuleMatchJSON<'a, 'b> {
       note: rule.note.clone(),
       message,
       labels,
+      metadata,
     }
   }
 }
@@ -247,6 +282,7 @@ pub struct JSONPrinter<W: Write> {
   output: W,
   style: JsonStyle,
   context: (u16, u16),
+  include_metadata: bool,
   // indicate if any matches happened
   matched: bool,
 }
@@ -262,6 +298,7 @@ impl<W: Write> JSONPrinter<W> {
     Self {
       style,
       output,
+      include_metadata: false,
       context: (0, 0),
       matched: false,
     }
@@ -271,6 +308,11 @@ impl<W: Write> JSONPrinter<W> {
     self.context = context;
     self
   }
+
+  pub fn include_metadata(mut self, include: bool) -> Self {
+    self.include_metadata = include;
+    self
+  }
 }
 
 impl<W: Write> Printer for JSONPrinter<W> {
@@ -281,6 +323,7 @@ impl<W: Write> Printer for JSONPrinter<W> {
     JSONProcessor {
       style: self.style,
       context: self.context,
+      include_metadata: self.include_metadata,
     }
   }
   fn process(&mut self, processed: Buffer) -> Result<()> {
@@ -329,6 +372,7 @@ impl<W: Write> Printer for JSONPrinter<W> {
 
 pub struct JSONProcessor {
   style: JsonStyle,
+  include_metadata: bool,
   context: (u16, u16),
 }
 
@@ -372,13 +416,13 @@ impl PrintProcessor<Buffer> for JSONProcessor {
   fn print_rule(
     &self,
     matches: Vec<NodeMatch>,
-    file: SimpleFile<Cow<str>, &String>,
+    file: SimpleFile<Cow<str>, &str>,
     rule: &RuleConfig<SgLang>,
   ) -> Result<Buffer> {
     let path = file.name();
     let jsons = matches
       .into_iter()
-      .map(|nm| RuleMatchJSON::new(nm, path, rule));
+      .map(|nm| RuleMatchJSON::new(nm, path, rule, self.include_metadata));
     self.print_docs(jsons)
   }
 
@@ -407,7 +451,7 @@ impl PrintProcessor<Buffer> for JSONProcessor {
     let path = path.to_string_lossy();
     let jsons = diffs
       .into_iter()
-      .map(|(diff, rule)| RuleMatchJSON::diff(diff, &path, rule));
+      .map(|(diff, rule)| RuleMatchJSON::diff(diff, &path, rule, self.include_metadata));
     self.print_docs(jsons)
   }
 }
@@ -453,7 +497,7 @@ mod test {
   }
 
   // source, pattern, replace, debug note
-  type Case<'a> = (&'a str, &'a str, &'a str, &'a str);
+  type Case<'t> = (&'t str, &'t str, &'t str, &'t str);
 
   const MATCHES_CASES: &[Case] = &[
     ("let a = 123", "a", "b", "Simple match"),
@@ -551,13 +595,12 @@ rule:
       if source.contains("import") {
         continue;
       }
-      let source = source.to_string();
       let mut printer = make_test_printer(JsonStyle::Pretty);
-      let grep = SgLang::from(SupportLang::Tsx).ast_grep(&source);
+      let grep = SgLang::from(SupportLang::Tsx).ast_grep(source);
       let rule = make_rule(pattern);
       let matches = grep.root().find_all(&rule.matcher).collect();
       printer.before_print().unwrap();
-      let file = SimpleFile::new(Cow::Borrowed("test.ts"), &source);
+      let file = SimpleFile::new(Cow::Borrowed("test.ts"), source);
       let buffer = printer
         .get_processor()
         .print_rule(matches, file, &rule)
@@ -667,4 +710,58 @@ transform:
     assert_eq!(metas.single["A"].text, "123");
     assert_eq!(metas.transformed["B"], "2");
   }
+
+  const LABEL_TEXT: &str = "
+labels:
+  A:
+    style: primary
+    message: var label
+";
+  #[test]
+  fn test_label() {
+    let mut printer = make_test_printer(JsonStyle::Compact);
+    let rule = get_rule_config(&format!("pattern: console.log($A)\n{}", LABEL_TEXT));
+    let grep = SgLang::from(SupportLang::TypeScript).ast_grep("console.log(123)");
+    let matches = grep.root().find_all(&rule.matcher).collect();
+    printer.before_print().unwrap();
+    let file = SimpleFile::new(Cow::Borrowed("test.ts"), "console.log(123)");
+    let buffer = printer
+      .get_processor()
+      .print_rule(matches, file, &rule)
+      .unwrap();
+    printer.process(buffer).unwrap();
+    printer.after_print().unwrap();
+    let json_str = get_text(&printer);
+    let json: Vec<RuleMatchJSON> = serde_json::from_str(&json_str).unwrap();
+    let labels = &json[0].labels;
+    assert_eq!(labels[0].message.unwrap(), "var label");
+    assert_eq!(labels[0].style, LabelStyle::Primary);
+    assert_eq!(labels[0].text, "123");
+  }
+
+  const META_TEXT: &str = "
+metadata:
+  A: test-meta
+";
+  #[test]
+  fn test_metadata() {
+    for included in [true, false] {
+      let mut printer = make_test_printer(JsonStyle::Compact).include_metadata(included);
+      let rule = get_rule_config(&format!("pattern: console.log($A)\n{}", META_TEXT));
+      let grep = SgLang::from(SupportLang::TypeScript).ast_grep("console.log(123)");
+      let matches = grep.root().find_all(&rule.matcher).collect();
+      printer.before_print().unwrap();
+      let file = SimpleFile::new(Cow::Borrowed("test.ts"), "console.log(123)");
+      let buffer = printer
+        .get_processor()
+        .print_rule(matches, file, &rule)
+        .unwrap();
+      printer.process(buffer).unwrap();
+      printer.after_print().unwrap();
+      let json_str = get_text(&printer);
+      let json: Vec<RuleMatchJSON> = serde_json::from_str(&json_str).unwrap();
+      let metadata = &json[0].metadata;
+      assert_eq!(metadata.is_some(), included);
+    }
+  }
 }
