@@ -10,6 +10,7 @@
 //!   * rules skipped (dues to ignore/files)
 //! - Detail level: show how a rule runs on a file
 
+use crate::config::ProjectConfig;
 use crate::lang::SgLang;
 use ast_grep_config::{RuleCollection, RuleConfig};
 
@@ -45,14 +46,24 @@ impl fmt::Debug for Granularity {
 }
 
 impl Granularity {
-  pub fn run_trace(&self) -> RunTrace {
-    self.run_trace_impl(std::io::stderr())
+  pub fn project_trace(&self) -> ProjectTrace {
+    self.project_trace_impl(std::io::stderr())
   }
-  fn run_trace_impl<W: Write>(&self, w: W) -> TraceInfo<(), W> {
+  fn project_trace_impl<W: Write>(&self, w: W) -> TraceInfo<(), W> {
     TraceInfo {
       level: *self,
       inner: (),
-      file_trace: Default::default(),
+      output: Mutex::new(w),
+    }
+  }
+
+  pub fn run_trace(&self) -> RunTrace {
+    self.run_trace_impl(std::io::stderr())
+  }
+  fn run_trace_impl<W: Write>(&self, w: W) -> TraceInfo<FileTrace, W> {
+    TraceInfo {
+      level: *self,
+      inner: Default::default(),
       output: Mutex::new(w),
     }
   }
@@ -64,7 +75,6 @@ impl Granularity {
     TraceInfo {
       level: *self,
       inner: rule_stats,
-      file_trace: Default::default(),
       output: Mutex::new(w),
     }
   }
@@ -89,7 +99,6 @@ impl FileTrace {
 
 pub struct TraceInfo<T, W: Write> {
   pub level: Granularity,
-  pub file_trace: FileTrace,
   pub inner: T,
   output: Mutex<W>,
 }
@@ -133,20 +142,33 @@ impl<T, W: Write + Sync> TraceInfo<T, W> {
     })
   }
 
-  fn print_files(&self) -> Result<()> {
+  fn print_files(&self, trace: &FileTrace) -> Result<()> {
     self.print_summary("file", |w| {
-      let scanned = self.file_trace.files_scanned.load(Ordering::Acquire);
-      let skipped = self.file_trace.files_skipped.load(Ordering::Acquire);
+      let scanned = trace.files_scanned.load(Ordering::Acquire);
+      let skipped = trace.files_skipped.load(Ordering::Acquire);
       write!(w, "scannedFileCount={scanned},skippedFileCount={skipped}")?;
       Ok(())
-    })?;
-    Ok(())
+    })
   }
 }
 
 impl<W: Write + Sync> TraceInfo<(), W> {
+  pub fn print_project(&self, project: &Result<ProjectConfig>) -> Result<()> {
+    self.print_summary("project", |w| {
+      if let Ok(project) = project {
+        let dir = project.project_dir.display();
+        write!(w, "isProject=true,projectDir={}", dir)?;
+      } else {
+        write!(w, "isProject=false")?;
+      }
+      Ok(())
+    })
+  }
+}
+
+impl<W: Write + Sync> TraceInfo<FileTrace, W> {
   pub fn print(&self) -> Result<()> {
-    self.print_files()
+    self.print_files(&self.inner)
   }
 
   pub fn print_file(&self, path: &Path, lang: SgLang) -> Result<()> {
@@ -160,7 +182,7 @@ impl<W: Write + Sync> TraceInfo<(), W> {
 impl<W: Write + Sync> TraceInfo<RuleTrace, W> {
   // TODO: support more format?
   pub fn print(&self) -> Result<()> {
-    self.print_files()?;
+    self.print_files(&self.inner.file_trace)?;
     self.print_summary("rule", |w| {
       let (effective, skipped) = (
         self.inner.effective_rule_count,
@@ -174,6 +196,7 @@ impl<W: Write + Sync> TraceInfo<RuleTrace, W> {
     })?;
     Ok(())
   }
+
   pub fn print_file(&self, path: &Path, lang: SgLang, rules: &[&RuleConfig<SgLang>]) -> Result<()> {
     self.print_entity("file", path.display(), |w| {
       let len = rules.len();
@@ -199,11 +222,13 @@ impl<W: Write + Sync> TraceInfo<RuleTrace, W> {
 
 #[derive(Default)]
 pub struct RuleTrace {
+  pub file_trace: FileTrace,
   pub effective_rule_count: usize,
   pub skipped_rule_count: usize,
 }
 
-pub type RunTrace = TraceInfo<(), Stderr>;
+pub type ProjectTrace = TraceInfo<(), Stderr>;
+pub type RunTrace = TraceInfo<FileTrace, Stderr>;
 pub type ScanTrace = TraceInfo<RuleTrace, Stderr>;
 
 #[cfg(test)]
@@ -216,14 +241,8 @@ mod test {
     let mut ret = String::new();
     let run_trace = tracing.run_trace_impl(unsafe { ret.as_mut_vec() });
     assert_eq!(run_trace.level, Granularity::Summary);
-    assert_eq!(
-      run_trace.file_trace.files_scanned.load(Ordering::Relaxed),
-      0
-    );
-    assert_eq!(
-      run_trace.file_trace.files_skipped.load(Ordering::Relaxed),
-      0
-    );
+    assert_eq!(run_trace.inner.files_scanned.load(Ordering::Relaxed), 0);
+    assert_eq!(run_trace.inner.files_skipped.load(Ordering::Relaxed), 0);
     assert!(run_trace.print().is_ok());
     assert_eq!(
       ret,
@@ -234,15 +253,24 @@ mod test {
     let rule_stats = RuleTrace {
       effective_rule_count: 10,
       skipped_rule_count: 2,
+      file_trace: Default::default(),
     };
     let scan_trace = tracing.scan_trace_impl(rule_stats, unsafe { ret.as_mut_vec() });
     assert_eq!(scan_trace.level, Granularity::Summary);
     assert_eq!(
-      scan_trace.file_trace.files_scanned.load(Ordering::Relaxed),
+      scan_trace
+        .inner
+        .file_trace
+        .files_scanned
+        .load(Ordering::Relaxed),
       0
     );
     assert_eq!(
-      scan_trace.file_trace.files_skipped.load(Ordering::Relaxed),
+      scan_trace
+        .inner
+        .file_trace
+        .files_skipped
+        .load(Ordering::Relaxed),
       0
     );
     assert_eq!(scan_trace.inner.effective_rule_count, 10);
