@@ -45,7 +45,7 @@ pub(super) fn match_node_impl<'tree, D: Doc>(
   }
 }
 
-fn match_nodes_impl_recursive<'tree, D: Doc + 'tree>(
+fn match_nodes_impl_recursive<'tree, D: Doc>(
   goals: &[PatternNode],
   candidates: impl Iterator<Item = Node<'tree, D>>,
   agg: &mut impl Aggregator<'tree, D>,
@@ -91,7 +91,7 @@ enum ControlFlow {
 }
 
 /// returns None means no match
-fn may_match_ellipsis_impl<'p, 't: 'p, D: Doc + 't>(
+fn may_match_ellipsis_impl<'p, 't: 'p, D: Doc>(
   goal_children: &mut Peekable<impl Iterator<Item = &'p PatternNode>>,
   cand_children: &mut Peekable<impl Iterator<Item = Node<'t, D>>>,
   agg: &mut impl Aggregator<'t, D>,
@@ -166,7 +166,7 @@ fn may_match_ellipsis_impl<'p, 't: 'p, D: Doc + 't>(
   }
 }
 
-fn match_single_node_while_skip_trivial<'p, 't: 'p, D: Doc + 't>(
+fn match_single_node_while_skip_trivial<'p, 't: 'p, D: Doc>(
   goal_children: &mut Peekable<impl Iterator<Item = &'p PatternNode>>,
   cand_children: &mut Peekable<impl Iterator<Item = Node<'t, D>>>,
   agg: &mut impl Aggregator<'t, D>,
@@ -239,13 +239,13 @@ mod test {
   use crate::{meta_var::MetaVarEnv, Matcher, Pattern, Root};
   use std::borrow::Cow;
   fn match_tree(p: &str, n: &str, strictness: MatchStrictness) -> MatchOneNode {
-    let pattern = Pattern::str(p, Tsx);
+    let pattern = Pattern::new(p, Tsx);
     let kind = pattern.potential_kinds().expect("should have kind");
     let kind = KindMatcher::from_id(kind.into_iter().next().expect("should have kind") as u16);
     let n = Root::str(n, Tsx);
     let n = n.root().find(kind).expect("should find");
     let mut env = Cow::Owned(MetaVarEnv::new());
-    match_node_impl(&pattern.node, &n, &mut env, &strictness)
+    match_node_impl(&pattern.node, &*n, &mut env, &strictness)
   }
   fn matched(p: &str, n: &str, strictness: MatchStrictness) {
     let ret = match_tree(p, n, strictness);
